from dataclasses import dataclass as __dataclass
from pymoodleclient.endpoints.base import Endpoint
from .course_responses import CourseList
from requests import get


@__dataclass
class GetAllCourses(Endpoint):
    """Get all courses from moodle.
    Courses appears in `Course` class"""

    response_object = CourseList
    request_type = get
    function_name = "core_course_get_courses"



from dataclasses import dataclass as __dataclass, field
from .endpoints.base import Endpoint


@__dataclass
class Params:
    wstoken: str
    wsfunction: Endpoint
    moodlewsrestformat: str = field(default="json")



from dataclasses import dataclass as __dataclass, InitVar
from typing import Optional

from pymoodleclient.endpoints.base import ResponseObject


@__dataclass
class CourseFormatOption:
    name: str
    value: int


@__dataclass
class Course:
    id: int
    shortname: str
    categoryid: int
    categorysortorder: str
    fullname: str
    displayname: str
    idnumber: int
    summary: str
    summaryformat: int
    format: str
    showgrades: int
    newsitems: int
    startdate: int
    enddate: int
    numsections: int
    maxbytes: int
    showreports: int
    visible: int
    groupmode: int
    groupmodeforce: int
    defaultgroupingid: int
    timecreated: int
    timemodified: int
    enablecompletion: int
    completionnotify: int
    lang: int
    forcetheme: int
    courseformatoptions: list[CourseFormatOption]

    def __post_init__(self):
        self.courseformatoptions = [
            CourseFormatOption(**course_format_option)
            for course_format_option in self.courseformatoptions
        ]


@__dataclass
class CourseList(ResponseObject):
    courses: list[Course]

    def __post_init__(self):
        for course in self.courses:
            if "hiddensections" in course.keys():
                course.pop("hiddensections")
            self.courses[self.courses.index(course)] = Course(**course)



from dataclasses import dataclass as __dataclass
from typing import Callable


@__dataclass
class Endpoint:
    """Base class of Endpoint Objects"""

    response_object: type
    request_type: Callable
    function_name: str



from .endpoints.base import Endpoint, ResponseObject
from .request_params import Params
from dataclasses import asdict


class Client:
    url: str
    token: str

    def __init__(self, url: str, token: str):
        self.url = url
        self.token = token

    def call(self, endpoint: Endpoint) -> ResponseObject:
        response: ResponseObject = endpoint.response_object(
            endpoint.request_type(
                f"{self.url}/webservice/rest/server.php",
                params=asdict(Params(self.token, endpoint.function_name)),
            ).json()
        )
        return response
